/* bayer.c
 *
 * Copyright  2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* This is from libgphoto2 */

#include <linux/kernel.h>
#include "sqcam.h"

static int tile_colors[8][4] = {
	{0, 1, 1, 2},
	{1, 0, 2, 1},
	{2, 1, 1, 0},
	{1, 2, 0, 1},
	{0, 1, 1, 2},
	{1, 0, 2, 1},
	{2, 1, 1, 0},
	{1, 2, 0, 1}};

#define RED 0
#define GREEN 1
#define BLUE 2

static int gp_bayer_expand (unsigned char *input, int w, int h, unsigned char *output, int tile)
{
	int x, y, i;
	int colour, bayer;
	char *ptr = input;

	switch (tile) {

		case 0:
		case 1:
		case 2:
		case 3: 

			for (y = 0; y < h; ++y)
				for (x = 0; x < w; ++x, ++ptr) {
					bayer = (x&1?0:1) + (y&1?0:2);

					colour = tile_colors[tile][bayer];

					i = (y * w + x) * 3;

					output[i+RED]    = 0;
					output[i+GREEN]  = 0;
					output[i+BLUE]   = 0;
					output[i+colour] = *ptr;
				}
			break;

		case 4: //BAYER_TILE_RGGB_INTERLACED:
		case 5: //BAYER_TILE_GRBG_INTERLACED:
		case 6: //BAYER_TILE_BGGR_INTERLACED:
		case 7: //BAYER_TILE_GBRG_INTERLACED:
 

			for (y = 0; y < h; ++y, ptr+=w)
				for (x = 0; x < w; ++x) {
					bayer = (x&1?0:1) + (y&1?0:2);
	
					colour = tile_colors[tile][bayer];
	
					i = (y * w + x) * 3;

					output[i+RED]    = 0;
					output[i+GREEN]  = 0;
					output[i+BLUE]   = 0;
					output[i+colour] = (x&1)? ptr[x>>1]:ptr[(w>>1)+(x>>1)];
				}
			break;
	}

	return (0);
}

#define AD(x, y, w) ((y)*(w)*3+3*(x))

static int
gp_bayer_interpolate (unsigned char *image, int w, int h, int tile)
{
	int x, y, bayer;
	int p0, p1, p2, p3;
	int div, value;


	switch (tile) {
	default:
	case 0: //BAYER_TILE_RGGB:
	case 4: //BAYER_TILE_RGGB_INTERLACED:
		p0 = 0; p1 = 1; p2 = 2; p3 = 3;
		break;
	case 1: //BAYER_TILE_GRBG:
	case 5: //BAYER_TILE_GRBG_INTERLACED:
		p0 = 1; p1 = 0; p2 = 3; p3 = 2;
		break;
	case 2: //BAYER_TILE_BGGR:
	case 6: //BAYER_TILE_BGGR_INTERLACED:
		p0 = 3; p1 = 2; p2 = 1; p3 = 0;
		break;
	case 3: //BAYER_TILE_GBRG:
	case 7: //BAYER_TILE_GBRG_INTERLACED:	
		p0 = 2; p1 = 3; p2 = 0; p3 = 1;
		break;
	}


	//p0 = 3; p1 = 2; p2 = 1; p3 = 0; // 2
	//p0 = 0; p1 = 1; p2 = 2; p3 = 3;


	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++) {
			bayer = (x&1?0:1) + (y&1?0:2);
			if ( bayer == p0 ) {
				
				/* red. green lrtb, blue diagonals */
				div = value = 0;
				if (y) {
					value += image[AD(x,y-1,w)+GREEN];
					div++;
				}
				if (y < (h - 1)) {
					value += image[AD(x,y+1,w)+GREEN];
					div++;
				}
				if (x) {
					value += image[AD(x-1,y,w)+GREEN];
					div++;
				}
				if (x < (w - 1)) {
					value += image[AD(x+1,y,w)+GREEN];
					div++;
				}
				image[AD(x,y,w)+GREEN] = value / div;

				div = value = 0;
				if ((y < (h - 1)) && (x < (w - 1))) {
					value += image[AD(x+1,y+1,w)+BLUE];
					div++;
				}
				if ((y) && (x)) {
					value += image[AD(x-1,y-1,w)+BLUE];
					div++;
				}
				if ((y < (h - 1)) && (x)) {
					value += image[AD(x-1,y+1,w)+BLUE];
					div++;
				}
				if ((y) && (x < (w - 1))) {
					value += image[AD(x+1,y-1,w)+BLUE];
					div++;
				}
				image[AD(x,y,w)+BLUE] = value / div;

			} else if (bayer == p1) {
				
				/* green. red lr, blue tb */
				div = value = 0;
				if (x < (w - 1)) {
					value += image[AD(x+1,y,w)+RED];
					div++;
				}
				if (x) {
					value += image[AD(x-1,y,w)+RED];
					div++;
				}
				image[AD(x,y,w)+RED] = value / div;

				div = value = 0;
				if (y < (h - 1)) {
					value += image[AD(x,y+1,w)+BLUE];
					div++;
				}
				if (y) {
					value += image[AD(x,y-1,w)+BLUE];
					div++;
				}
				image[AD(x,y,w)+BLUE] = value / div;

			} else if ( bayer == p2 ) {
				
				/* green. blue lr, red tb */
				div = value = 0;
				
				if (x < (w - 1)) {
					value += image[AD(x+1,y,w)+BLUE];
					div++;
				}
				if (x) {
					value += image[AD(x-1,y,w)+BLUE];
					div++;
				}
				image[AD(x,y,w)+BLUE] = value / div;

				div = value = 0;
				if (y < (h - 1)) {
					value += image[AD(x,y+1,w)+RED];
					div++;
				}
				if (y) {
					value += image[AD(x,y-1,w)+RED];
					div++;
				}
				image[AD(x,y,w)+RED] = value / div;

			} else {
				
				/* blue. green lrtb, red diagonals */
				div = value = 0;

				if (y) {
					value += image[AD(x,y-1,w)+GREEN];
					div++;
				}
				if (y < (h - 1)) {
					value += image[AD(x,y+1,w)+GREEN];
					div++;
				}
				if (x) {
					value += image[AD(x-1,y,w)+GREEN];
					div++;
				}
				if (x < (w - 1)) {
					value += image[AD(x+1,y,w)+GREEN];
					div++;
				}
				image[AD(x,y,w)+GREEN] = value / div;

				div = value = 0;
				if ((y < (h - 1)) && (x < (w - 1))) {
					value += image[AD(x+1,y+1,w)+RED];
					div++;
				}
				if ((y) && (x)) {
					value += image[AD(x-1,y-1,w)+RED];
					div++;
				}
				if ((y < (h - 1)) && (x)) {
					value += image[AD(x-1,y+1,w)+RED];
					div++;
				}
				if ((y) && (x < (w - 1))) {
					value += image[AD(x+1,y-1,w)+RED];
					div++;
				}
				image[AD(x,y,w)+RED] = value / div;
			}
		}

	return (0);
}

static int
gp_bayer_decode (unsigned char *input, int w, int h, unsigned char *output,
		 int tile)
{
	gp_bayer_expand (input, w, h, output, tile);
	gp_bayer_interpolate (output, w, h, tile);

	return (0);
}

static int gamma_correct_triple( 
		unsigned char *table_red,
		unsigned char *table_green,
		unsigned char *table_blue,
		unsigned char *data, unsigned int size )
{
	int x;

	for (x = 0; x < (size * 3); x += 3) {
		data[x + 0] = table_red  [data[x + 0]];
		data[x + 1] = table_green[data[x + 1]];
		data[x + 2] = table_blue [data[x + 2]];
	}

	return (0);
}

static int
gamma_correct_single (unsigned char *table, unsigned char *data,
			 unsigned int size)
{
	return (gamma_correct_triple (table, table, table, data, size));
}

static int gamma_fill_table (unsigned char *table, double g)
{
/*	int x;

	for (x = 0; x < 256; x++)

		table[x] = 255 * pow ((double) x/255., g);
*/
	return (0);
}

#define DATA_HEADER_SIZE 64

void sqcam_decode_bayer(const u8 *data, u8 *rgb, struct sqcam *cam)
{
	int j;
	unsigned char b;
	unsigned char *src;
	unsigned char *dst;

	src = (u8 *) data + DATA_HEADER_SIZE;
	dst = rgb;

	/* Reverse the data, else the pic is upside down. */
	for (j = 0; j < 320*240 / 2; j++) {
		b = src[j];
		src[j] = src[320*240 - 1 - j];
		src[320*240 - 1 - j] = b;
	}
	gp_bayer_decode(src, 320, 240, dst, cam->channel);
	/*gamma_correct_single (gamma_table, dst  , 320 * 240);*/
}															 

