/*
**	Christophe Lucas <clucas@rotomalug.org>
**
**	This file is released under GPL terms.
*/

#include "sqcam.h"

static struct sensor_t sq905;


static int sensor_sq905_init(struct sqcam *cam)
{
	int status=0;
	unsigned char ch[5], zh;
	struct usb_device* udev = cam->udev ;

	printk(KERN_INFO "SQCam driver: Initializing camera..." );
	
	/* See sqcam.txt : 3.1 Initialization */
	zh = 0x00;
	status +=  send_control_msg(SEND, cam, 0x0c, 0x06, 0xf0, &zh, 1);
	zh = 0x00;
	status += send_control_msg(RECEIVE, cam, 0x0c, 0x07, 0, &zh, 1);
	ch[0] = 0x00;
	zh = 0x00;
	status += send_control_msg(SEND, cam, 0x0c, 0x06, 0xa0, &zh, 1);
	zh = 0x00;
	status += send_control_msg(RECEIVE, cam, 0x0c, 0x07, 0, &zh, 1);
#ifdef V4L1
    	cam->channel = 1;
#endif
	if (status)
		printk(KERN_INFO "failed :-(\n");
	else
		printk(KERN_INFO "done :-)\n");
	return status;
}

static int sensor_sq905_reset(struct sqcam *cam)
{
	int status;
	unsigned char ch;

	printk(KERN_INFO "SQCam driver: Resetting camerea.\n");

	ch = 0xff;
	if ((status = send_control_msg(SEND, cam, 0x0c, 0xc0, 0x00, &ch, 1)) < 0)
		return status;
	if ((status = send_control_msg(SEND, cam, 0x0c, 0x06, 0xa0, NULL, 0)) < 0)
		return status;
	if ((status = send_control_msg(SEND, cam, 0x0c, 0x07, 0, &ch, 1)) < 0)
		return status;
	return 0;
}

int sensor_sq905_probe(struct sqcam* cam)
{
	sqcam_attach_sensor(cam, &sq905);	

	if (le16_to_cpu(sq905.udev->descriptor.idProduct) != 0x9120 )
		return -ENODEV;

	dev_info(&cam->udev->dev, "SQ905 sensor detected.\n");
	return 0;
}

static struct sensor_t sq905 = {
	.init = &sensor_sq905_init;
	.reset = &sensor_sq905_reset;
};

