#ifndef _SQCAM_H_
#define _SQCAM_H_

#include <linux/kref.h>
#include <asm/semaphore.h>

#include "sensor.h"

#define SQCAM_DRIVER_MAJORVERSION         0
#define SQCAM_DRIVER_MINORVERSION         2

#define SQCAM_DRIVER_VERSION __stringify(SQCAM_DRIVER_MAJORVERSION) "." \
                            __stringify(SQCAM_DRIVER_MINORVERSION)

#define SQCAM_BYTES_PER_PIXEL 	3
#define SQCAM_MAX_READ_SIZE 	0x8000
#define SQCAM_MAX_RAW_SIZE 	320*240+64
#define SQCAM_MAX_FRAME_SIZE 	(SQCAM_BYTES_PER_PIXEL*320*240)
#define SQCAM_FRAMES 		2
#define SHARPNESS_MIN   	0
#define SHARPNESS_MAX   	6
#define FRAMERATE_MIN   	0
#define FRAMERATE_MAX   	6

#define USB_SQCAM_VENDOR_ID		0x2770
#define USB_SQCAM_SQ905_SENSOR_ID	0x9120	
#define USB_SQCAM_HV7131R_SENSOR_ID 	0x905c

SQCAM_TABLE_SENSORS

/* Structure to hold all of our device specific stuff */
struct sqcam {
	struct video_device 	*vdev;			/* the video device for this device */
	struct usb_device 	*udev;			/* the usb device for this device */
	struct usb_interface 	*interface;		/* the interface for this device */
	unsigned char 		*bulk_in_buffer;		/* the buffer to receive data */
	size_t			bulk_in_size;		/* the size of the receive buffer */
	__u8			bulk_in_endpointAddr;	/* the address of the bulk in endpoint */
	__u8			bulk_out_endpointAddr;	/* the address of the bulk out endpoint */
	struct kref		kref;
	int			is_initialized;
	int			channel ;
        __u32			framebuf_size;		/* of valid bytes in framebuf */
	__u32			framebuf_read_start;	/* position in frame buf read */
        __u8 			*raw_image;		/* raw data captured from the camera */
	__u8 			*framebuf;		/* processed data in RGB24 format */
	__u16			reg[32];		/* device register */
		
	/* prevent from race condition */
	struct semaphore	cam_lock ;

	struct sensor_t*	sensor;
};

/* ------------------------------------------------------------------- */
void sqcam_attach_sensor(struct sqcam* cam, struct sensor_t* sensor )
{
	cam->sensor = sensor;
	cam->sensor->udev = cam->udev;
}

#endif

